// Name
// Student ID

import fisica.*;

FWorld world;

// A constant that determines the sizes of the pool
// balls
int ball_diameter = 20;
// Keep track of the cue ball, so that we can figure
// out when the user clicks on it.
FCircle cue_ball;

// The usual flag from direct manipulation interfaces.
boolean dragging;

// A useful helper function.  Given the (x,y) coordinates
// and (r,g,b) colour of a pool ball, create the ball
// and add it to the physical simulation.  Returns the
// created FCircle in case you need to refer to it.
FCircle makeCircle( float x, float y, 
  float r, float g, float b )
{
  FCircle circ = new FCircle( ball_diameter );
  circ.setPosition( x, y );
  circ.setFill( r, g, b );
  circ.setRestitution( 0.9 );
  world.add( circ );
  circ.setGrabbable( false );
  return circ;
}

// A helper function that tells you if any pool
// balls are currently touching one of your pocket
// detectors.  Pass in the detector as input and
// you'll get back an array of FBody objects, each
// of which is presumably one ball in that pocket.
FBody[] getCollidingBalls( FBody detector )
{
  ArrayList al = detector.getContacts();
  FBody[] ret = new FBody[ al.size() ];
  int idx = 0;
  for( Object o : al ) {
    FContact con = (FContact)o;
    FBody other = con.getBody1();
    if( other == detector ) {
      other = con.getBody2();
    }
    ret[idx] = other;
    ++idx;
  }
  return ret;
}

void setup()
{
  size( 600, 341 );
  
  // Set up the Fisica environment.
  Fisica.init( this );
  world = new FWorld();

  // Create the cue ball.
  cue_ball = makeCircle( 150, 170, 255, 255, 255 );
}

void draw()
{
  // Advance the physical simulation.
  world.step();
  
  // Render the Fisica world
  background( 0 );
  world.draw();

  // If you're currently lining up a shot, show the
  // line from the cue ball to the mouse.
  if ( dragging ) {
    stroke( 255, 255, 0 );
    strokeWeight( 3.0 );
    line( cue_ball.getX(), cue_ball.getY(), mouseX, mouseY );
  }
}

// Check if the user clicked on the cue ball.  If so,
// start lining up a shot.  You don't need to change
// anything in here.
void mousePressed()
{
  FBody pressed = world.getBody( mouseX, mouseY );
  if ( pressed == cue_ball ) {
    dragging = true;
  }
}

// When the user lets go of the mouse, if they were 
// lining up a shot then actually make that shot by
// giving the cue ball a bunch of velocity.  You don't
// need to change anything in here.
void mouseReleased()
{
  if ( dragging ) {
    dragging = false;
    float vx = 10.0 * (cue_ball.getX() - mouseX);
    float vy = 10.0 * (cue_ball.getY() - mouseY);

    cue_ball.setVelocity( vx, vy );
  }
}